import subprocess
from dateutil.parser import parse
from datetime import datetime
import xarray
import io
import os
import numpy as np
import typing
import importlib.resources

iri_name = "iri2016_driver"
if os.name == "nt":
    iri_name += ".exe"

if not importlib.resources.is_resource(__package__, iri_name):
    with importlib.resources.path(__package__, "setup.cmake") as setup:
        subprocess.check_call(["ctest", "-S", str(setup), "-VV"])

SIMOUT = ["ne", "Tn", "Ti", "Te", "nO+", "nH+", "nHe+", "nO2+", "nNO+", "nCI", "nN+"]

__all__ = ["IRI"]


def IRI(time: datetime, altkmrange: typing.Sequence[float], glat: float, glon: float) -> xarray.Dataset:

    if isinstance(time, str):
        time = parse(time)

    assert len(altkmrange) == 3, "altitude (km) min, max, step"
    assert isinstance(glat, float) and isinstance(glon, float), "glat, glon is scalar"

    with importlib.resources.path(__package__, iri_name) as exe:
        cmd = [
            str(exe),
            str(time.year),
            str(time.month),
            str(time.day),
            str(time.hour),
            str(time.minute),
            str(time.second),
            str(glat),
            str(glon),
            str(altkmrange[0]),
            str(altkmrange[1]),
            str(altkmrange[2]),
            str(exe.parent / "data"),
        ]

        ret = subprocess.check_output(cmd, text=True)  # str for Windows
    # %% get altitude profile data
    Nalt = int((altkmrange[1] - altkmrange[0]) // altkmrange[2]) + 1

    arr = np.genfromtxt(io.StringIO(ret), max_rows=Nalt)
    arr = np.atleast_2d(arr)
    assert arr.ndim == 2 and arr.shape[1] == 12, "bad text data output format"

    dsf = {k: (("alt_km"), v) for (k, v) in zip(SIMOUT, arr[:, 1:].T)}
    altkm = arr[:, 0]
    # %% get parameter data
    arr = np.genfromtxt(io.StringIO(ret), skip_header=Nalt)
    assert arr.ndim == 1 and arr.size == 100, "bad text data output format"
    # %% assemble output
    iono = xarray.Dataset(
        dsf, coords={"time": [time], "alt_km": altkm, "glat": glat, "glon": glon}, attrs={"f107": arr[40], "ap": arr[51]}
    )

    for i, p in enumerate(["NmF2", "hmF2", "NmF1", "hmF1", "NmE", "hmE"]):
        iono[p] = (("time"), [arr[i]])

    iono["TEC"] = (("time"), [arr[36]])
    iono["EqVertIonDrift"] = (("time"), [arr[43]])
    iono["foF2"] = (("time"), [arr[99]])

    return iono



import iri2016 as iri

from pathlib import Path
from argparse import ArgumentParser
from datetime import timedelta

try:
    from matplotlib.pyplot import show
    import iri2016.plots as piri
except ImportError:
    show = piri = None


def altitude():
    """ Height Profile Example """

    if show is None:
        raise ImportError("need matplotlib")

    p = ArgumentParser(description="IRI2016 altitude profile")
    p.add_argument("time", help="time of simulation")
    p.add_argument("latlon", help="geodetic latitude, longitude (degrees)", type=float, nargs=2)
    p.add_argument("-alt_km", help="altitude START STOP STEP (km)", type=float, nargs=3, default=(80, 1000, 10))
    P = p.parse_args()

    iono = iri.IRI(P.time, P.alt_km, *P.latlon)

    piri.altprofile(iono)
    show()


def latitude():
    """ latitude Profile Example """

    if show is None:
        raise ImportError("need matplotlib")

    p = ArgumentParser(description="IRI2016 latitude profile")
    p.add_argument("glon", help="geodetic  longitude (degrees)", type=float)
    p.add_argument("-glat", help="geodetic latitude START STOP STEP (degrees)", type=float, nargs=3, default=(-60, 60, 2.0))
    p.add_argument("-alt_km", help="altitude (km)", type=float, default=300.0)
    p.add_argument("-o", "--outfn", help="write data to file")
    P = p.parse_args()

    iono = iri.geoprofile(latrange=P.glat, glon=P.glon, altkm=P.alt_km, time="2004-01-01T17")

    if P.outfn:
        outfn = Path(P.outfn).expanduser()
        print("writing", outfn)
        iono.to_netcdf(outfn)

    piri.latprofile(iono)
    show()


def time_profile():
    """ IRI2016 time profile """

    if show is None:
        raise ImportError("need matplotlib")

    p = ArgumentParser()
    p.add_argument("time", help="start yy-mm-dd, stop yy-mm-dd, step_hour", nargs=3)
    p.add_argument("latlon", help="geodetic latitude, longitude (degrees)", nargs=2, type=float)
    p.add_argument("-alt_km", help="altitude START STOP STEP (km)", type=float, nargs=3, default=(100, 200, 20))
    P = p.parse_args()

    sim = iri.timeprofile((P.time[0], P.time[1]), timedelta(hours=float(P.time[2])), P.alt_km, *P.latlon)

    piri.timeprofile(sim)
    show()



#!/usr/bin/env python3
import setuptools
import site

# PEP517 workaround
site.ENABLE_USER_SITE = True

setuptools.setup()

"""
Because of bugs in IRI2016 itself, present even in plain Fortran usage, we can't safely use F2py, bad data can result.
"""
